@@ -25,6 +25,7 @@
 #endif
 
 #include "data_p.h"
+#include "context_p.h"
 #include <error.h>
 #include <interfaces/dataprovider.h>
 
@@ -47,11 +48,11 @@ GpgME::Data::Data()
 {
     gpgme_data_t data;
     const gpgme_error_t e = gpgme_data_new(&data);
-    d.reset(new Private(e ? 0 : data));
+    d.reset(new Private(e ? nullptr : data));
 }
 
 GpgME::Data::Data(const Null &)
-    : d(new Private(0))
+    : d(new Private(nullptr))
 {
 
 }
@@ -69,14 +70,14 @@ GpgME::Data::Data(const char *buffer, size_t size, bool copy)
     std::string sizestr = std::to_string(size);
     // Ignore errors as this is optional
     gpgme_data_set_flag(data, "size-hint", sizestr.c_str());
-    d.reset(new Private(e ? 0 : data));
+    d.reset(new Private(e ? nullptr : data));
 }
 
 GpgME::Data::Data(const char *filename)
 {
     gpgme_data_t data;
     const gpgme_error_t e = gpgme_data_new(&data);
-    d.reset(new Private(e ? 0 : data));
+    d.reset(new Private(e ? nullptr : data));
     if (!e) {
         setFileName(filename);
     }
@@ -85,29 +86,29 @@ GpgME::Data::Data(const char *filename)
 GpgME::Data::Data(const char *filename, off_t offset, size_t length)
 {
     gpgme_data_t data;
-    const gpgme_error_t e = gpgme_data_new_from_filepart(&data, filename, 0, offset, length);
-    d.reset(new Private(e ? 0 : data));
+    const gpgme_error_t e = gpgme_data_new_from_filepart(&data, filename, nullptr, offset, length);
+    d.reset(new Private(e ? nullptr : data));
 }
 
 GpgME::Data::Data(FILE *fp)
 {
     gpgme_data_t data;
     const gpgme_error_t e = gpgme_data_new_from_stream(&data, fp);
-    d.reset(new Private(e ? 0 : data));
+    d.reset(new Private(e ? nullptr : data));
 }
 
 GpgME::Data::Data(FILE *fp, off_t offset, size_t length)
 {
     gpgme_data_t data;
-    const gpgme_error_t e = gpgme_data_new_from_filepart(&data, 0, fp, offset, length);
-    d.reset(new Private(e ? 0 : data));
+    const gpgme_error_t e = gpgme_data_new_from_filepart(&data, nullptr, fp, offset, length);
+    d.reset(new Private(e ? nullptr : data));
 }
 
 GpgME::Data::Data(int fd)
 {
     gpgme_data_t data;
     const gpgme_error_t e = gpgme_data_new_from_fd(&data, fd);
-    d.reset(new Private(e ? 0 : data));
+    d.reset(new Private(e ? nullptr : data));
 }
 
 GpgME::Data::Data(DataProvider *dp)
@@ -117,20 +118,20 @@ GpgME::Data::Data(DataProvider *dp)
         return;
     }
     if (!dp->isSupported(DataProvider::Read)) {
-        d->cbs.read = 0;
+        d->cbs.read = nullptr;
     }
     if (!dp->isSupported(DataProvider::Write)) {
-        d->cbs.write = 0;
+        d->cbs.write = nullptr;
     }
     if (!dp->isSupported(DataProvider::Seek)) {
-        d->cbs.seek = 0;
+        d->cbs.seek = nullptr;
     }
     if (!dp->isSupported(DataProvider::Release)) {
-        d->cbs.release = 0;
+        d->cbs.release = nullptr;
     }
     const gpgme_error_t e = gpgme_data_new_from_cbs(&d->data, &d->cbs, dp);
     if (e) {
-        d->data = 0;
+        d->data = nullptr;
     }
     if (dp->isSupported(DataProvider::Seek)) {
         off_t size = seek(0, SEEK_END);
@@ -216,6 +217,11 @@ GpgME::Error GpgME::Data::setFileName(const char *name)
     return Error(gpgme_data_set_file_name(d->data, name));
 }
 
+GpgME::Error GpgME::Data::setFileName(const std::string &name)
+{
+    return Error(gpgme_data_set_file_name(d->data, name.c_str()));
+}
+
 ssize_t GpgME::Data::read(void *buffer, size_t length)
 {
     return gpgme_data_read(d->data, buffer, length);
@@ -230,3 +236,47 @@ off_t GpgME::Data::seek(off_t offset, int whence)
 {
     return gpgme_data_seek(d->data, offset, whence);
 }
+
+GpgME::Error GpgME::Data::rewind()
+{
+    return Error(gpgme_data_rewind(d->data));
+}
+
+std::vector<GpgME::Key> GpgME::Data::toKeys(Protocol proto) const
+{
+    std::vector<GpgME::Key> ret;
+    if (isNull()) {
+        return ret;
+    }
+    auto ctx = GpgME::Context::createForProtocol(proto);
+    if (!ctx) {
+        return ret;
+    }
+
+    if (gpgme_op_keylist_from_data_start (ctx->impl()->ctx, d->data, 0)) {
+        return ret;
+    }
+
+    gpgme_key_t key;
+    while (!gpgme_op_keylist_next (ctx->impl()->ctx, &key)) {
+        ret.push_back(GpgME::Key(key, false));
+    }
+    gpgme_data_seek (d->data, 0, SEEK_SET);
+
+    delete ctx;
+    return ret;
+}
+
+std::string GpgME::Data::toString()
+{
+  std::string ret;
+  char buf[4096];
+  size_t nread;
+  seek (0, SEEK_SET);
+  while ((nread = read (buf, 4096)) > 0)
+    {
+      ret += std::string (buf, nread);
+    }
+  seek (0, SEEK_SET);
+  return ret;
+}
